package formdef.plugin.validator.digester;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.apache.commons.digester.Digester;
import org.xml.sax.SAXException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.validator.ValidatorResources;
import formdef.plugin.validator.digester.rules.SetNextUnlessAttrRule;


/**
 * Configures the Validator engine from an XML file containing
 * both FormDef and Validator data.
 */
public class ValidatorResourcesInitializer {

    /**
     * Commons Logging instance.
     */
    protected static Log log = LogFactory.getLog(ValidatorResourcesInitializer.class);


    /**
     * The set of public identifiers, and corresponding resource names, for
     * the versions of the configuration file DTDs that we know about.  
     */
    protected static String registrations[] = {
        "-//Apache Software Foundation//DTD Commons Validator Rules Configuration 1.0//EN",
        "/org/apache/commons/validator/resources/validator_1_0.dtd",
        "-//Apache Software Foundation//DTD Commons Validator Rules Configuration 1.0.1//EN",
        "/org/apache/commons/validator/resources/validator_1_0_1.dtd",
        "-//Apache Software Foundation//DTD Commons Validator Rules Configuration 1.1//EN",
        "/org/apache/commons/validator/resources/validator_1_1.dtd",
        "-//Apache Software Foundation//DTD Commons Validator Rules Configuration 1.1.3//EN",
        "/org/apache/commons/validator/resources/validator_1_1_3.dtd",
        "-//FormDef//FormDef Form Definition and Validation//EN",
        "/formdef/plugin/validator/form-defs_0_5-validator_1_0.dtd",
        "-//FormDef//Combined FormDef Form Definition 0.5 and Validation 1.1.3//EN",
        "/formdef/plugin/validator/form-defs_0_5-validator_1_1_3.dtd",
        "-//FormDef//Combined FormDef Form Definition 0.5 and Validation 1.2.0//EN",
        "/formdef/plugin/validator/form-defs_0_5-validator_1_2_0.dtd",
        "-//FormDef//Combined FormDef Form Definition 1.0 and Validation 1.1.3//EN",
        "/formdef/plugin/validator/form-defs_1_0-validator_1_1_3.dtd",
        "-//FormDef//Combined FormDef Form Definition 1.0 and Validation 1.2.0//EN",
        "/formdef/plugin/validator/form-defs_1_0-validator_1_2_0.dtd"
    };


    /**
     * Initializes a <code>ValidatorResources</code> based on the <code>InputStream</code>
     * and processes the resources based on the <code>boolean</code> passed in.
     *
     * @param	resources	Resources to initialize.
     * @param	in		<code>InputStream</code> for the xml resource.
     * @param	process		Whether or not to call process on <code>ValidatorResources</code>.
     */
    public static void initialize(ValidatorResources resources, InputStream in, boolean process)
            throws IOException {

        Digester digester = new Digester();
        digester.push(resources);
        digester.setNamespaceAware(true);
        digester.setValidating(true);
        
        // register DTDs
        for (int i = 0; i < registrations.length; i += 2) {
            URL url = ValidatorResourcesInitializer.class.getResource(registrations[i + 1]);
            if (url != null) {
                digester.register(registrations[i], url.toString());
            }
        }

        digester.setUseContextClassLoader(true);

        //**********************************************************************
        // Rules coming from the original ValidatorResourcesInitializer
        
        // Create Global Constant objects
        digester.addCallMethod("form-validation/global/constant",
                "addConstant", 2);
        digester.addCallParam("form-validation/global/constant/constant-name", 0);
        digester.addCallParam("form-validation/global/constant/constant-value", 1);

        // Create Global ValidatorAction objects
        digester.addObjectCreate("form-validation/global/validator",
                "org.apache.commons.validator.ValidatorAction", "className");
        digester.addSetProperties("form-validation/global/validator");
        digester.addSetNext("form-validation/global/validator", "addValidatorAction",
                "org.apache.commons.validator.ValidatorAction");

        // Add the body of a javascript element to the Validatoraction
        digester.addCallMethod("form-validation/global/validator/javascript",
                "setJavascript", 0);

        
        // Create FormSet objects
        digester.addObjectCreate("form-validation/formset", "org.apache.commons.validator.FormSet",
                "className");
        digester.addSetProperties("form-validation/formset");
        digester.addSetNext("form-validation/formset", "addFormSet",
                "org.apache.commons.validator.FormSet");

        // Create Constant objects
        digester.addCallMethod("form-validation/formset/constant",
                "addConstant", 2);
        digester.addCallParam("form-validation/formset/constant/constant-name", 0);
        digester.addCallParam("form-validation/formset/constant/constant-value", 1);

        // Create Form objects
        digester.addObjectCreate("form-validation/formset/form",
                "org.apache.commons.validator.Form", "className");
        digester.addSetProperties("form-validation/formset/form");
        digester.addSetNext("form-validation/formset/form", "addForm",
                "org.apache.commons.validator.Form");
        // Create Field objects
        digester.addObjectCreate("form-validation/formset/form/field",
                "org.apache.commons.validator.Field", "className");
        digester.addSetProperties("form-validation/formset/form/field");
        digester.addSetNext("form-validation/formset/form/field", "addField",
                "org.apache.commons.validator.Field");

        // Create Var objects
        digester.addCallMethod("form-validation/formset/form/field/var",
                "addVar", 3);
        digester.addCallParam("form-validation/formset/form/field/var/var-name", 0);
        digester.addCallParam("form-validation/formset/form/field/var/var-value", 1);
        digester.addCallParam("form-validation/formset/form/field/var/var-jstype", 2);
        
        // Create Msg object
        digester.addObjectCreate("form-validation/formset/form/field/msg",
                "org.apache.commons.validator.Msg", "className");
        digester.addSetProperties("form-validation/formset/form/field/msg");
        digester.addSetNext("form-validation/formset/form/field/msg", "addMsg",
                "org.apache.commons.validator.Msg");
      		    
        // Create Arg objects
        // Arg
        digester.addObjectCreate("form-validation/formset/form/field/arg",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-validation/formset/form/field/arg");
        digester.addSetNext("form-validation/formset/form/field/arg", "addArg",
                "org.apache.commons.validator.Arg");
      
        // Arg0
        digester.addObjectCreate("form-validation/formset/form/field/arg0",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-validation/formset/form/field/arg0");
        digester.addSetNext("form-validation/formset/form/field/arg0", "addArg0",
                "org.apache.commons.validator.Arg");
      
        // Arg1
        digester.addObjectCreate("form-validation/formset/form/field/arg1",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-validation/formset/form/field/arg1");
        digester.addSetNext("form-validation/formset/form/field/arg1", "addArg1",
                "org.apache.commons.validator.Arg");
      
        // Arg2
        digester.addObjectCreate("form-validation/formset/form/field/arg2",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-validation/formset/form/field/arg2");
        digester.addSetNext("form-validation/formset/form/field/arg2", "addArg2",
                "org.apache.commons.validator.Arg");
      
        // Arg3
        digester.addObjectCreate("form-validation/formset/form/field/arg3",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-validation/formset/form/field/arg3");
        digester.addSetNext("form-validation/formset/form/field/arg3", "addArg3",
                "org.apache.commons.validator.Arg");
      

        //**********************************************************************
        // Rules to load definitions from a combined FormDef/Validator xml
        
        // Create FormSet objects
        digester.addObjectCreate("form-definition/formset", "org.apache.commons.validator.FormSet",
                "className");
        digester.addSetProperties("form-definition/formset");
        digester.addSetNext("form-definition/formset", "addFormSet",
                "org.apache.commons.validator.FormSet");

        // Create Constant objects
        digester.addCallMethod("form-definition/formset/constant",
                "addConstant", 2);
        digester.addCallParam("form-definition/formset/constant/constant-name", 0);
        digester.addCallParam("form-definition/formset/constant/constant-value", 1);

        // Create Form objects
        digester.addObjectCreate("form-definition/formset/form",
                "org.apache.commons.validator.Form", "className");
        digester.addSetProperties("form-definition/formset/form");
        digester.addRule("form-definition/formset/form",
                new SetNextUnlessAttrRule("addForm",
                        "org.apache.commons.validator.Form","validate","false"));
        //digester.addSetNext("form-definition/formset/form", "addForm",
        //        "org.apache.commons.validator.Form");

        // Create Field objects
        digester.addObjectCreate("form-definition/formset/form/field",
                "org.apache.commons.validator.Field", "className");
        digester.addSetProperties("form-definition/formset/form/field");
        digester.addRule("form-definition/formset/form/field",
                new SetNextUnlessAttrRule("addField",
                        "org.apache.commons.validator.Field","validate","false"));
        //digester.addSetNext("form-definition/formset/form/field", "addField",
        //        "org.apache.commons.validator.Field");

        // Create Var objects
        digester.addCallMethod("form-definition/formset/form/field/var",
                "addVar", 3);
        digester.addCallParam("form-definition/formset/form/field/var/var-name", 0);
        digester.addCallParam("form-definition/formset/form/field/var/var-value", 1);
        digester.addCallParam("form-definition/formset/form/field/var/var-jstype", 2);

        // Create Msg object
        digester.addObjectCreate("form-definition/formset/form/field/msg",
                "org.apache.commons.validator.Msg", "className");
        digester.addSetProperties("form-definition/formset/form/field/msg");
        digester.addSetNext("form-definition/formset/form/field/msg", "addMsg",
                "org.apache.commons.validator.Msg");

        // Create Arg objects
        // Arg
        digester.addObjectCreate("form-definition/formset/form/field/arg",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-definition/formset/form/field/arg");
        digester.addSetNext("form-definition/formset/form/field/arg", "addArg",
                "org.apache.commons.validator.Arg");

        // Arg0
        digester.addObjectCreate("form-definition/formset/form/field/arg0",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-definition/formset/form/field/arg0");
        digester.addSetNext("form-definition/formset/form/field/arg0", "addArg0",
                "org.apache.commons.validator.Arg");

        // Arg1
        digester.addObjectCreate("form-definition/formset/form/field/arg1",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-definition/formset/form/field/arg1");
        digester.addSetNext("form-definition/formset/form/field/arg1", "addArg1",
                "org.apache.commons.validator.Arg");

        // Arg2
        digester.addObjectCreate("form-definition/formset/form/field/arg2",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-definition/formset/form/field/arg2");
        digester.addSetNext("form-definition/formset/form/field/arg2", "addArg2",
                "org.apache.commons.validator.Arg");

        // Arg3
        digester.addObjectCreate("form-definition/formset/form/field/arg3",
                "org.apache.commons.validator.Arg", "className");
        digester.addSetProperties("form-definition/formset/form/field/arg3");
        digester.addSetNext("form-definition/formset/form/field/arg3", "addArg3",
                "org.apache.commons.validator.Arg");

        try {
            // Parse the input stream to initialize our database
            digester.parse(in);

        } catch (SAXException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (in != null) {
                in.close();
            }
        }

        if (process) {
            resources.process();
        }

    }

}
