
package formdef.plugin.validator.digester.rules;

import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.digester.Rule;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.xml.sax.Attributes;


/**
 * <p>An alternative <code>SetNextRule</code> implementation that passes the
 * top object (child) to the top-1 object (parent) unless a named attribute
 * does not have a specified value.  It can be used to block a parent-child
 * relationship.</p>
 */
public class SetNextUnlessAttrRule extends Rule {


    private static final Log log =
            LogFactory.getLog(SetNextUnlessAttrRule.class);

    // ----------------------------------------------------------- Constructors



    /**
     * Construct a "set next unless attr" rule with the specified method name,
     * param type, attribute name and given value.
     *
     * @param methodName the method to call when the conditions allow it
     * @param paramType the parameter type for the method
     * @param attributeName the name of the attribute whose value will
     *          be checked
     * @param givenValue the String value that the attribute will
     *          be checked for
     */
    public SetNextUnlessAttrRule(
            String methodName,
            String paramType,
            String attributeName,
            String givenValue) {

        this.methodName = methodName;
        this.paramType = paramType;
        this.attributeName = attributeName;
        this.givenValue = givenValue;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * The method name to call on the parent object.
     */
    protected String methodName = null;

    /**
     * The Java class name of the parameter type expected by the method.
     */
    protected String paramType = null;

    /**
     * The name of the attribute whose value will be checked.
     */
    protected String attributeName = null;

    /**
     * The value to check equality against.
     */
    protected String givenValue = null;

    /**
     * Holds the actual value for the named attribute.
     */
    protected String actualValue = null;


    // --------------------------------------------------------- Public Methods



    /**
     * This method is called when the beginning of a matching XML element
     * is encountered. The default implementation delegates to the deprecated
     * method {@link #begin(org.xml.sax.Attributes) begin} without the
     * <code>namespace</code> and <code>name</code> parameters, to retain
     * backwards compatibility.
     *
     * @param namespace the namespace URI of the matching element, or an
     *   empty string if the parser is not namespace aware or the element has
     *   no namespace
     * @param name the local name if the parser is namespace aware, or just
     *   the element name otherwise
     * @param attributes The attribute list of this element
     * @since Digester 1.4
     */
    public void begin(String namespace, String name, Attributes attributes)
            throws Exception {
        for (int i=0; i < attributes.getLength(); i++) {
            //log.debug("\nAttribute: Local name=" + attributes.getLocalName(i)
            //    + " qName=" + attributes.getQName(i)
            //    + " type=" + attributes.getType(i)
            //    + " value=" + attributes.getValue(i));
            if (attributeName.equals(attributes.getLocalName(i))) {
                //log.debug("MATCH with localName");
                actualValue = attributes.getValue(i);
                //if (givenValue.equals(attributes.getValue(i))) {
                //    log.debug("MATCHED VALUE!");
                //}
            } else if (attributeName.equals(attributes.getQName(i))) {
                //log.debug("MATCH with qName");
                actualValue = attributes.getValue(i);
                //if (givenValue.equals(attributes.getValue(i))) {
                //    log.debug("MATCHED VALUE!");
                //}
            }
        }

        super.begin(namespace, name, attributes);
    }

    /**
     * Process the end of this element.
     */
    public void end(String namespace, String name) throws Exception {

        // Identify the objects to be used
        Object child = digester.peek(0);
        Object parent = digester.peek(1);

        if (!givenValue.equals(actualValue)) {
            // Call the specified method
            Class paramTypes[] = new Class[1];
            if (paramType != null) {
                paramTypes[0] =
                        digester.getClassLoader().loadClass(paramType);
            } else {
                paramTypes[0] = child.getClass();
            }

            MethodUtils.invokeMethod(parent, methodName,
                new Object[]{ child }, paramTypes);
        } else {
            if (log.isDebugEnabled()) {
                log.debug("parent method not invoked because "
                    + "\"" + givenValue + "\".equals(\"" + actualValue + "\")");
            }
        }
        
        actualValue = null;
    }


    /**
     * Render a printable version of this Rule.
     */
    public String toString() {

        StringBuffer sb = new StringBuffer(256);
        sb.append("SetNextUnlessAttrRule[");
        sb.append("methodName=");
        sb.append(methodName);
        sb.append(", paramType=");
        sb.append(paramType);
        sb.append("attributeName=");
        sb.append(attributeName);
        sb.append(", givenValue=");
        sb.append(givenValue);
        sb.append("]");
        return (sb.toString());

    }


}
